
(*  Module  : MakeLink
    Author  : Franz Schwarz
    Address : Muehlenstrasse 2
    Address : D-7201 Durchhausen
    Address : Germany
    email   : FIDO: Franz Schwarz, 2:241/7506.18
    email   : uucp: Franz_Schwarz@p18.f7506.n241.z2.fidonet.org
    Date    : 24-Apr-92
    Language: Oberon
    Compiler: Amiga Oberon V2.14
    Type    : Small CLI utility
    Legal   : Released to the Public Domain
    Required: AmigaOS 2.04 or higher
    Info    : This is just a substitute for Commodore's Workbench 2.0/2.1
    Info    : MakeLink command. It works as Commodore's was designed to
    Info    : work, offering complete compatability to CBM's Makelink,
    Info    : The HARD/s switch is processed as opposed to CBM, where
    Info    : it was just ignored and all links were HARD ones, which
    Info    : amounted to its inability of creating SOFT links although
    Info    : they're supported by the operating system. In fact, if you
    Info    : neglect to specify HARD, my Makelink will always create
    Info    : soft links, which have the enourmous advantage of beeing
    Info    : permitted to *reference *accross dos devices!*. You could
    Info    : e.g. type 'makelink DH0:c/LHARC DH1:c/LHA, and you could
    Info    : access dh0:c/lharc which would cause the DOS to take
    Info    : DH1:c/LHA for operation! BTW: Soft links WILL survive a
    Info    : cold start. Oh, not to forget: If you want to make a link to
    Info    : a directory you must specify the FORCE/S option, as CBM's
    Info    : makelink expects it, too! Note that a WB start of
    Info    : will cause this program to terminate without any action.
    Info    :
    Problems: None known.
    History : 24-Apr-92 Version 38.1, initial public release.
 *)

(* thoroughly tested, thus: *)
(* $OvflChk- $RangeChk- $StackChk- $NilChk- $ReturnChk- $CaseChk- *)

(* BTW: You should use SmalCode/SmallData to enable residentability *)

MODULE MakeLink;

IMPORT d: Dos, e: Exec, o: OberonLib, SYS: SYSTEM;

CONST tmpl = "FROM/A,TO/A,HARD/S,FORCE/S"; (* FORCE retained for CBM compatability *)
      version = "\000$VER: Oberon-MakeLink 38.1 (24-Apr-92)";

PROCEDURE main(): BOOLEAN;

VAR ArgArray: ARRAY 4 OF e.APTR;
  rda: d.RDArgsPtr;
  a,b: e.STRPTR;
  l: d.FileLockPtr;
  fib: d.FileInfoBlockPtr;
  success: BOOLEAN;
  dummy: LONGINT;
BEGIN
  (* persuade Oberon to keep the version string in the executable *)
  IF SYS.ADR(version)#NIL THEN END;
  success:=FALSE;
  IF (d.dos.lib.version >= 37) THEN
    dummy:=d.SetIoErr(0);
    ArgArray[2]:=NIL; (* clear HARD & FORCE switch *)
    ArgArray[3]:=NIL;
    rda := d.ReadArgs (tmpl, ArgArray, NIL);
    IF rda#NIL THEN
      a:=ArgArray[0];
      b:=ArgArray[1];
      l:=d.Lock(b^, d.sharedLock);
      IF l#NIL THEN
        fib:=d.AllocDosObject(d.fib,NIL);
        IF fib#NIL THEN
          IF d.Examine(l,fib^) THEN
            IF ~((fib.dirEntryType > 0) & (ArgArray[3]=NIL)) THEN
                  (* make sure that FORCE is specified if we link a dir *)
              IF ArgArray[2]#NIL THEN (* Hard Link *)
                IF d.MakeLink(a^,SYS.VAL(LONGINT,l),d.DOSFALSE)#d.DOSFALSE THEN
                  success:=TRUE
                ELSE
                  dummy:=d.Printf("MakeLink failed");
                END;
              ELSE (* Soft Link *)
                IF d.MakeLink(a^,SYS.VAL(LONGINT,b),d.DOSTRUE)#d.DOSFALSE THEN
                  success:=TRUE
                ELSE
                  dummy:=d.Printf("MakeLink failed");
                END;
              END;
            ELSE
              dummy:=d.Printf("Links to Dirs require the FORCE switch");
              dummy:=d.SetIoErr(d.objectWrongType);
            END;
          ELSE
            dummy:=d.Printf("Failed to Examine the lock");
          END;
          d.FreeDosObject(d.fib,fib);
        ELSE
          dummy:=d.Printf("Can't allocate DOS object");
          dummy:=d.SetIoErr(d.noFreeStore);
        END;
        d.UnLock(l);
      ELSE
        dummy:=d.Printf("No lock for %s",b);
      END;
      d.FreeArgs(rda);
    ELSE
      dummy:=d.Printf("Args parse error");
    END;
    IF ~success THEN
      IF d.IoErr()#0 THEN
        IF d.PrintFault(d.IoErr(),"") THEN END;
      ELSE
        dummy:=d.Printf("\n");
      END;
    END;
  END;
  RETURN success;
END main;

BEGIN
  IF o.wbStarted THEN
    HALT(20);
  ELSIF main() THEN
    HALT(0);
  ELSE
    HALT(20);
  END;
END MakeLink.















