MODULE umsConsts;

IMPORT  Utility, sys:SYSTEM;

CONST

   NumFields *	= 128;

   (* enumeration of fields in an UMS-message *)

   msgText *	=  0;
   fromName *	=  1;
   fromPath *	=  2;
   toName *	=  3;
   toPath *	=  4;
   msgID *	=  5;
   creationDate*=  6;
   receiveDate*	=  7;
   refID *	=  8;
   group * 	=  9;
   subject * 	= 10;
   attributes *	= 11;
   comments *	= 12;
   organisation*= 13;
   distribution*= 14;

TYPE
   STRPTR* = POINTER TO ARRAY MAX(INTEGER)-1 OF CHAR;
   MsgTextFields* = ARRAY NumFields OF STRPTR;
   MessageInfo*	= STRUCT
                    hdrLen*	: LONGINT;
                    txtLen*	: LONGINT;
                    date*	: LONGINT;
                    up*, dn*,
                    lt*, rt*	: LONGINT;
                    globalStat*	: LONGSET;
                    userStat*	: LONGSET;
                    loginStat*	: LONGSET;
                    hardLink*	: LONGINT;
                    softLink*	: LONGINT;
                  END;
CONST
  EmptyMsgInfo*	= MessageInfo(0,0,0,0,0,0,0,LONGSET{},LONGSET{},LONGSET{},0,0);

  (* user status-bits *)

  archive *   = 4;  (* msg should be archived, don't delete *)
  junk *      = 5;  (* negative selection, inheritance suggested *)
  postPoned * = 6;  (* to be read again, (but not now) *)
  selected *  = 7;  (* positive selection, inheritance suggested *)

  Old *		=  8; (* user has already read this Message *)
  Read *	=  Old;
  WriteAccess *	=  9; (* user may change or delete this message *)
  ReadAccess *	= 10; (* user may read this message *)
  ViewAccess *	= 11; (* user may read the header of this message *)
  Owner *	= 12; (* user 'owns' (wrote) this message *)

  ProtectedUserFlags*= {WriteAccess,ReadAccess,ViewAccess,Owner};

  (* global status-bits *)

  Deleted *	= 0; (* msg is really deleted *)
  Expired *	= 1; (* msg has expired and may be deleted *)
  Exported *	= 2; (* msg has been exported *)
  Orphan *	= 3; (* msg could not be exported *)
  Link *	= 4; (* within a ring of linked msgs *)
  HardLink *	= 5; (* link is hardLink *)

  ProtectedGlobalFlags*= {Deleted,Exported,Orphan,Link,HardLink};

(*** Errors ***)

CONST
  ok*		=  0;
  unknown*	=  1;

  codeMissing*	= 100;
  forbiddenCode*= 101;
  noWriteAccess*= 102;
  noReader*	= 103;
  noExporter*	= 104;
  badLink*	= 105;
  noWork*	= 106;
  noSysop*	= 107;

  dupe*		= 200;
  noReadAccess* = 201;
  noViewAccess* = 202;
  msgCorrupted* = 203;
  noHdrSpace*	= 204;
  noSuchMsg*	= 205;
  badName*	= 206;
  badTag*	= 207;
  missingTag*	= 208;
  noSuchUser*	= 209;
  notFound*	= 210;
  autoBounce*	= 211;
  msgDeleted*	= 212;
  noNetAccess*	= 213;

  serverTerminated*= 300;
  cantWrite*	= 301;
  cantRead* 	= 302;
  wrongMsgPtr*  = 303;
  serverNotFree*= 304;
  idCountProb*	= 305;
  noLogin*	= 306;
  wrongServer*	= 307;

TYPE
  ErrTxt* = STRUCT num*:LONGINT; str*:STRPTR END;
  ErrArr* = ARRAY 32 OF ErrTxt;

CONST
  ErrorText*= ErrArr(
    noNetAccess,	sys.ADR("no net-access"),
    msgDeleted,		sys.ADR("msg is deleted"),
    wrongServer,	sys.ADR("wrong Server Version"),
    noSysop,		sys.ADR("you're not a sysop"),
    autoBounce,		sys.ADR("bad msg (to be bounced) forwarded to sysop"),
    noWork,		sys.ADR("no work to do"),
    noSuchUser,		sys.ADR("not such user"),
    notFound,		sys.ADR("not found"),
    serverTerminated,	sys.ADR("server has terminated"),
    missingTag,		sys.ADR("required tag missing"),
    badTag,		sys.ADR("unknown or forbidden tag"),
    noLogin,		sys.ADR("you're not logged in"),
    noExporter,		sys.ADR("you're no exporter"),
    unknown, 		sys.ADR("unknown"),
    codeMissing,	sys.ADR("Code missing"),
    forbiddenCode,	sys.ADR("forbidden Code"),
    noWriteAccess, 	sys.ADR("no write-access"),
    cantWrite,		sys.ADR("write error"),
    dupe,		sys.ADR("dupe"),
    noReadAccess,	sys.ADR("no read-access"),
    noViewAccess,	sys.ADR("not visible"),
    cantRead,		sys.ADR("read error"),
    msgCorrupted,	sys.ADR("message is corrupted"),
    wrongMsgPtr,	sys.ADR("msg-ptr does not fit"),
    noHdrSpace,		sys.ADR("no space in header"),
    serverNotFree,	sys.ADR("server is not free"),
    idCountProb,	sys.ADR("idCount error"),
    noSuchMsg,		sys.ADR("msg does not exist"),
    badName,		sys.ADR("bad name or alias"),
    badLink,		sys.ADR("link error"),
    noReader,		sys.ADR("no one will read your msg"),
    ok, 		sys.ADR("ok"));

CONST

  (* tag-values *)

  (* you still MUST add Utility.user (TAG_USER) to the following values *)

  (* indicate, what type TagItem.data is of *)
  (* tagXY                            -> 'LONGINT'/'LONG'               *)
  (* tagXY              + typeVARPAR  -> 'POINTER TO LONGINT'/'LONG *'  *)
  (* tagXY + typeSTRPTR               -> 'STRPTR'/'char *'              *)
  (* tagXY + typeSTRPTR + typeVARPAR  -> 'POINTER TO STRPTR'/'char **'  *)

  typeSTRPTR *		= 2000H;
  typeVARPAR *		= 4000H;


  (** tags for ReadUMSMsg() and WriteUMSMsg() **)

  (*  add 'typeVARPAR' for ReadUMSMsg()  *)
  (*  no 'typeVARPAR' for WriteUMSMsg()  *)

  tagMsgNum * 		=  1;

  tagHdrLength *	=  2;
  tagTxtLength *	=  3;
  tagMsgDate *		=  4;   (* don't usually use when writing!      *)
                                (* Using tagMsgDate with WriteUMSMsg()  *)
                                (* has a very special meaning.          *)
  tagChainUp *		=  7;
  tagChainDn *		=  8;
  tagChainLt *		=  9;
  tagChainRt *		= 10;
  tagGlobalFlags *	= 11;
  tagUserFlags *	= 12;
  tagLoginFlags *	= 13;
  tagHardLink *		= 14;
  tagSoftLink *		= 15;

  tagDateStyle *	= 64;	(* style for receiveDate when reading:		*)
				(* data = 0: no receiveDate			*)
				(* data = 1: emulate old-style receiveDate	*)
  tagAutoBounce *	= 65;	(* when writing:				*)
				(* data # 0: server returns error 'autoBounce',	*)
				(*           but still keeps the Msg and sends	*)
				(*           it to the sysops.			*)
  tagHdrFill *		= 66;   (* when writing: how much bytes to be reserved	*)
  tagTxtFill *		= 67;	(*               for header and text		*)
  tagIDStyle *		= 68;	(* style for Message-ID				*)
 				(* data = 0: real Message-ID			*)
                                (* data = 1: old style dec-number		*)
  tagNoUpdate *		= 69;	(* data = 0: (default) set 'Old'-Flag when      *)
				(*           reading or writing			*)
				(* data = 1: don't touch 'Old'-Flag		*)

  tagMsgText *		=  256 + typeSTRPTR;
  tagFromName *		= tagMsgText + 1;
  tagFromPath *		= tagMsgText + 2;
  tagToName *		= tagMsgText + 3;
  tagToPath *		= tagMsgText + 4;
  tagMsgID *		= tagMsgText + 5;
  tagCreationDate *	= tagMsgText + 6;
  tagReceiveDate *	= tagMsgText + 7;
  tagRefID *		= tagMsgText + 8;
  tagGroup *		= tagMsgText + 9;
  tagSubject *		= tagMsgText +10;
  tagAttributes *	= tagMsgText +11;
  tagComments *		= tagMsgText +12;
  tagOrganisation *	= tagMsgText +13;
  tagDistribution *	= tagMsgText +14;

  (* tagMsgText+15 .. tagMsgText+127 are also usable as text-fields *)

  tagMsgInfo *		= 512;
				(* datatype: POINTER TO MessageInfo *)
  tagTextFields *	= tagMsgInfo + 1;
				(* datatype: POINTER TO MsgTextFields *)

  tagReadHeader *	= tagMsgInfo + 2;
				(* read all header-fields *)
  tagReadAll *		= tagMsgInfo + 3;
				(* read all text-fields *)


  (** tags for UMSSelect()  **)

  tagSelSet *		= 1024;
				(* flags to set on selected msgs *)
  tagSelUnset *		= tagSelSet + 1;
				(* flags to clear *)
  tagSelWriteGlobal *	= tagSelSet + 2;
				(* change global flags, not user-flags *)
  tagSelWriteLocal *	= tagSelSet + 3;
				(* change login-local flags, not user-flags *)
  tagSelWriteUser *	= tagSelSet + 4+typeSTRPTR;
				(* change other user's flags *)

  tagSelStart *		= tagSelSet + 8;
				(* process only msgs AFTER this one *)
  tagSelStop *		= tagSelSet + 9;
				(* process only msgs BEFORE this one *)

  (* the following are mutual-exclusiv *)
  (*   use only one of the following operations,   *)
  (*   otherwise unpredictable things may happen!  *)

  tagSelReadGlobal *	= tagSelSet +10;
				(* examine global flags, not user-flags *)
  tagSelReadLocal *	= tagSelSet +11;
				(* examine login-local flags, not user-flags *)
  tagSelReadUser *	= tagSelSet +12+typeSTRPTR;
				(* examine other user's flags *)
  tagSelMask *		= tagSelSet +16;
				(* select msgs, that's flags    *)
  tagSelMatch *		= tagSelSet +17;
				(*   ANDed with mask equal match *)
  tagSelParent *	= tagSelSet +18;
				(* examine parent's flags *)

  tagSelDate *		= tagSelSet +19;
				(* select msgs younger than this date *)

  tagSelTree *		= tagSelSet +29;
				(* select whole tree this msg is in *)

  tagSelSubTree *	= tagSelSet +21;
				(* select sub-tree this msg is root of *)

  tagSelMsg *		= tagSelSet +22;
				(* select a msg specified by number *)

  tagSelQuick *		= tagSelSet +23;
				(* quick select enabled *)
  (* tagMsgText .. tagMsgText+127 are also allowed for selecting *)


  (** tags for UMSSearch() **)

  tagSearchLast *	= 2048;
				(* number of LAST msg not to search *)
  tagSearchQuick *	= tagSearchLast+ 1;
				(* quick searches enabled *)

  tagSearchGlobal *	= tagSearchLast+ 2;
				(* examine global flags instead of user-flags*)
  tagSearchLocal *	= tagSearchLast+ 3;
				(* examine login-local flags, not user-flags *)
  tagSearchUser *	= tagSearchLast+ 4+typeSTRPTR;
				(* examine other user's flags *)
  tagSearchDirection *	= tagSearchLast+ 5;
				(* set search direction *)
				(*  0 = default  *)
				(*  1 = forward  *)
				(* -1 = backward *)

  (* the following are mutual-exclusiv *)
  (*   use only one of the following operations,   *)
  (*   otherwise unpredictable things may happen!  *)

  tagSearchMask *	= tagSearchLast+16;
				(* search a msg, that's flags   *)
  tagSearchMatch *	= tagSearchLast+17;
				(*   ANDed with mask equal match *)

  (* tagMsgText .. tagMsgText+127 are also allowed for searching *)


  (**  tags for ReadUMSConfig(), WriteUMSConfig()  **)

  tagCfgGlobalOnly *	= 3072; (* read only global config,                  *)
  				(* thus must not be combined with tagCfgUser *)
  tagCfgName *		= tagCfgGlobalOnly+ 1 + typeSTRPTR;
  				(* name of config var to read                *)
  tagCfgUser *		= tagCfgGlobalOnly+ 2 + typeSTRPTR;
  				(* name of user to read locals from          *)
  tagCfgUserName *	= tagCfgGlobalOnly+ 3 + typeSTRPTR;
  				(* alias to get realname from                *)


  (**  tags for WriteUMSConfig()  **)

  tagCfgDump *		= tagCfgGlobalOnly+16 + typeSTRPTR;
  				(* write current settings to a file *)


END umsConsts.
