MODULE TrapList;

(**********************************************************************)
(*                Traplist.library support definitions                *)
(*   for use with the Oberon compiler, by Fridtjof Siebert  A+L AG   *)
(*              Done by Roger Nordin (2:203/507@fidonet)              *)
(*    Updated for Oberon >2.14 by Kai Bolay (2:247/706.3@fidonet)     *)
(*   Please send corrections/bug reports regarding this file to me!   *)
(*                                                                    *)
(*                   This version created 17-Aug-92                   *)
(*                                                                    *)
(*   Based on nl.h (02 Apr 1991),  Copyright by Martin J. Laubach.   *)
(**********************************************************************)

IMPORT
  e: Exec, I: Intuition, y: SYSTEM;

CONST
  traplistName * = "traplist.library";
  traplistVersion * = 4;

  verbatim     * = LONGSET{31}; (* Don't translate phone numbers, used by FindNL() *)

  parseInvalidChar   * = -1; (* Returned by NLParseAddr() *)
  parseInvalidNumber * = -2;

  (* defined/reserved Tag IDs *)

  PASS * = y.VAL (LONGINT, "PASS"); (* password entry   *)
  TCNF * = y.VAL (LONGINT, "TCNF"); (* TD config entry  *)
  ACCT * = y.VAL (LONGINT, "ACCT"); (* accounting entry *)

TYPE
  NodeListPtr * = y.ADDRESS;
  ExtraPtr * = y.ADDRESS;

  AddrPtr * = UNTRACED POINTER TO Addr;
  Addr * = STRUCT
    zone  * : INTEGER;
    net   * : INTEGER;
    node  * : INTEGER;
    point * : INTEGER;
  END;

  (* This structure is returned by NLFind() and NLIndexFind() *)

  NodeDescPtr * = UNTRACED POINTER TO NodeDesc;
  NodeDesc * = STRUCT
    node      * : Addr;    (* node address *)
    reserved  * : LONGINT;
    region    * : INTEGER; (* region (0 if none) *)
    hubNode   * : INTEGER; (* node # of this node's HUB (0 if none) *)
    system    * : e.STRPTR; (* node name *)
    sysop     * : e.STRPTR; (* sysop name *)
    phone     * : e.STRPTR; (* phone number *)
    city      * : e.STRPTR; (* city and state *)
    passwd    * : e.STRPTR; (* password *)
    flags     * : e.STRPTR; (* nodelist flags *)
    cost      * : e.STRPTR; (* cost *)
    baudRate  * : INTEGER; (* baud rate *)
    id        * : LONGINT; (* unique ID (offset into nodeindex) *)
  END;

  ExtraEnumNodeProc* = PROCEDURE (userdata: y.ADDRESS; address: AddrPtr): BOOLEAN;
  ExtraEnumTagProc* = PROCEDURE (userdata: y.ADDRESS; size: LONGINT; address: AddrPtr): BOOLEAN;
  EnumNodeProc* = PROCEDURE (userdata: y.ADDRESS; region: LONGINT; address: AddrPtr): BOOLEAN;

VAR
  traplist * : e.LibraryPtr;

PROCEDURE Open* {traplist,-36}(name{8}:ARRAY OF CHAR; flags{0}:LONGSET):NodeListPtr;
PROCEDURE Close* {traplist,-42}(nodelist{8}:NodeListPtr);
PROCEDURE Find* {traplist,-48}(nodelist{8}:NodeListPtr; address{9}:AddrPtr; flags{0}:LONGSET):NodeDescPtr;
PROCEDURE FreeNode* {traplist,-54}(node{8}:NodeDescPtr);
PROCEDURE AddrComp* {traplist,-60}(node1{8},node2{9}:AddrPtr):SHORTINT;
PROCEDURE IndexFind* {traplist,-66}(nodelist{8}:NodeListPtr;address{9}:AddrPtr; flags{0}:LONGSET):NodeDescPtr;
PROCEDURE ExtraFromNL* {traplist,-72}(nodelist{8}:NodeListPtr):ExtraPtr;
PROCEDURE ExtraOpen* {traplist,-78}(path{8}:ARRAY OF CHAR):ExtraPtr;
PROCEDURE ExtraClose* {traplist,-84}(xtra{8}:ExtraPtr);
PROCEDURE ExtraFind* {traplist,-90}(xtra{8}:ExtraPtr; address{9}:AddrPtr; tag{0}:LONGINT; data{10}:y.ADDRESS):y.ADDRESS;
PROCEDURE ExtraPut* {traplist,-96}(xtra{8}:ExtraPtr; address{9}:AddrPtr; tag{0}:LONGINT; size{1}:LONGINT; data{10}:y.ADDRESS):BOOLEAN;
PROCEDURE ExtraDel* {traplist,-102}(xtra{8}:ExtraPtr; data{9}:y.ADDRESS);
PROCEDURE ExtraFlush* {traplist,-114}(xtra{8}:ExtraPtr);
PROCEDURE ExtraLock* {traplist,-120}(xtra{8}:ExtraPtr);
PROCEDURE ExtraUnlock* {traplist,-126}(xtra{8}:ExtraPtr);
PROCEDURE ExtraEnumNode* {traplist,-132}(xtra{8}:ExtraPtr;client{9}:ExtraEnumNodeProc; user{10}:y.ADDRESS):LONGINT;
PROCEDURE ExtraChanged* {traplist,-138}(xtra{8}:ExtraPtr);
PROCEDURE ExtraEnumTag* {traplist,-144}(xtra{8}:ExtraPtr; address{9}:AddrPtr; client{10}:ExtraEnumTagProc; user{11}:y.ADDRESS):LONGINT;
PROCEDURE EnumNode* {traplist,-150}(nodelist{8}:NodeListPtr; bufsize{0}:LONGINT; address{9}:AddrPtr; client{10}:EnumNodeProc; user{11}:y.ADDRESS):LONGINT;
PROCEDURE ParseAddr* {traplist,-156}(addr{8}:AddrPtr; str{9}:ARRAY OF CHAR; default{10}:AddrPtr):LONGINT;
PROCEDURE ExtraLockTag* {traplist,-168}(xtra{8}:ExtraPtr; tagdata{9}:y.ADDRESS);
PROCEDURE ExtraUnlockTag* {traplist,-174}(xtra{8}:ExtraPtr; tagdata{9}:y.ADDRESS);
PROCEDURE ExtraChange* {traplist,-180}(xtra{8}:ExtraPtr; address{9}:AddrPtr; tag{0}:LONGINT; size{1}:LONGINT; data{2}:y.ADDRESS):BOOLEAN;
PROCEDURE ExtraFindLock* {traplist, -186}(xtra{8}:ExtraPtr; address{9}:AddrPtr; tag{0}:LONGINT; size{10}:LONGINT):y.ADDRESS;

BEGIN
  traplist := e.OpenLibrary(traplistName,traplistVersion);
  IF traplist = NIL THEN
    y.SETREG (0, I.DisplayAlert (I.recoveryAlert,"\x00\x64\x14missing traplist.library v4\o\o",50));
    HALT(20);
  END;
CLOSE
  IF traplist#NIL THEN e.CloseLibrary(traplist); traplist := NIL END;
END TrapList.
