(*************************************************************************

:Program.    MemPoolTest.mod
:Contents.   test module for MemPool
:Author.     Hartmut Goebel [hG]
:Copyright.  Public Domain
:Language.   Oberon
:Translator. Amiga Oberon V2.25
:History.    V1.0, 28 May 1992 [hG]
:Date.       28 May 1992 18:23:06

:Support.    3nd test based on [bne] ClusterTest (AMOK # 24)

*************************************************************************)

MODULE MemPoolTest;

IMPORT
  e  := Exec,
  mp := MemPool,
  rnd:= Random,
  s := SYSTEM;

CONST
  Max = 600;
  MaxMemStep = 1024;
  NumCicles = 20000;

VAR
  a1, a2, a3: e.APTR;
  i, j, k: INTEGER;
  Test : ARRAY Max OF s.ADDRESS;

BEGIN
(* $IF test1 *)
  (*
   * 1st test: only to look if it works
   *)
  mp.New(a1,512); (* 1st block *)
  mp.New(a2,512);
  mp.New(a3,512); (* 2nd block *)

  mp.Dispose(a3); (* 2nd should be freed *)
  mp.Dispose(a2);

  mp.New(a3,512); (* should come from 1st block *);
  mp.Dispose(a1);
(* $END *)

(* $IF test2 *)
  (*
   * 2nd test: stress memory a bit more
   *)
  FOR i := 0 TO Max-1 DO
    k := rnd.RND(MaxMemStep)+1;
    mp.New(Test[i], k);
  END;
  FOR i := 0 TO Max-1 DO
    mp.Dispose(Test[i]);
  END;
(* $END *)

(* $IF test3 *)
  (*
   * 3rd test: stress memory evoen more
   *)
  FOR i := 0 TO Max-1 DO
    Test[i] :=NIL;
  END;

  FOR j := 0 TO NumCicles DO
    i := rnd.RND(Max);
    IF Test[i] = NIL THEN
      k := rnd.RND(MaxMemStep)+1;
      mp.New(Test[i], k);
      IF Test[i] = NIL THEN HALT(10); END;
    ELSE
      mp.Dispose(Test[i]);
    END;
  END;
(* $END *)

END MemPoolTest.
