(* -------------------------------------------------------------------------
  :Program.       IfLink
  :Contents.      Traces a given Link and writes the original FileName
  :Contents.      into "t:MyLink".
  :Remark.        -----------------
  :Remark.        Should set Returncode Warn, if File/Dir was a link,
  :Remark.        I don't know why, but I always get DirEntryType -3, so
  :Remark.        one has to compare the LinkName and the Original Name
  :Remark.        to see, if it was a Link.
  :Remark.        So, use it at Your Own Risk!
  :Remark.        -----------------
  :Author.        Werner Speer
  :Address.       Buchenstr. 3
  :Phone.         8508 Wendelstein
  :Copyright.     Freeware
  :Language.      Modula
  :Translator.    M2 Amiga V3.32
  :Imports.       MemSystem
------------------------------------------------------------------------- *)

MODULE IfLink;

FROM FileSystem IMPORT Response;

IMPORT
       Arguments,
       Arts,
       Dos,
       InOut,
       MemSystem,
       Str,
       SYSTEM,
       FileSystem;

CONST linkName = "T:myLink";

VAR myLock : Dos.FileLockPtr;


PROCEDURE CleanUp;
BEGIN
  IF myLock # NIL THEN
    Dos.UnLock(myLock);
    myLock := NIL;
  END;
END CleanUp;


PROCEDURE CheckLink;
VAR num,len : INTEGER;
    arg : ARRAY [0..30] OF CHAR;
    success : BOOLEAN;
    myInfoPtr : Dos.FileInfoBlockPtr;
    flag : BOOLEAN;
    entry : LONGINT;
    fileName : ARRAY [0..32] OF CHAR;
    file : FileSystem.File;
    dosLib : Dos.DosLibraryPtr;
    act : LONGINT;

BEGIN
  num := Arguments.NumArgs();

  dosLib := SYSTEM.ADR(Dos);
  IF dosLib^.lib.version <37 THEN
    InOut.WriteString("You need AmigaOS 2.x !");InOut.WriteLn;
    Arts.returnVal := 40;
    Arts.Terminate(Arts.CurrentLevel());
  END;

  IF num = 0 THEN
    InOut.WriteString("IfLink : Filename missing");InOut.WriteLn;
    Arts.returnVal := Dos.error;
    Arts.Terminate(Arts.CurrentLevel());
  END;

  IF num > 0 THEN
    Arguments.GetArg(1,arg,len);
    IF arg[0] = "?" THEN
      InOut.WriteString("usage  :");InOut.WriteLn;
      InOut.WriteString("  IfLink <filename>");InOut.WriteLn;
      InOut.WriteString("result :");InOut.WriteLn;
      InOut.WriteString("  Returncode 0     if File/Dir");InOut.WriteLn;
      InOut.WriteString("  (Returncode warn  if Link; DOES NOT WORK!)");InOut.WriteLn;
      InOut.WriteString("  Returncode error if no name was given");InOut.WriteLn;
      InOut.WriteString("  Returncode fail  if File/Dir was not found");InOut.WriteLn;
      InOut.WriteString("  Returncode 40    if not Kickstart 2.x");InOut.WriteLn;InOut.WriteLn;
      InOut.WriteString("  Writes the original FileName into 'T:MyLink'");InOut.WriteLn;InOut.WriteLn;
    ELSE
      myLock := Dos.Lock(SYSTEM.ADR(arg),Dos.sharedLock);
      IF myLock # NIL THEN
        MemSystem.Allocate(myInfoPtr,SIZE(Dos.FileInfoBlock));
        success := Dos.Examine(myLock,myInfoPtr);
        IF success THEN
          entry := myInfoPtr^.dirEntryType;

          InOut.WriteString("dirEntryType = ");
          InOut.WriteInt(myInfoPtr^.dirEntryType,2);
          InOut.WriteLn;
          (*
          InOut.WriteString("EntryType = ");
          InOut.WriteInt(myInfoPtr^.entryType,2);
          InOut.WriteLn;
          *)

          InOut.WriteString("Original filename = ");
          InOut.WriteString(myInfoPtr^.fileName);InOut.WriteLn;

          Str.Copy(fileName,myInfoPtr^.fileName);
          MemSystem.Deallocate(myInfoPtr);

          (*
          flag := FALSE;
          IF entry = -4 THEN
            InOut.WriteString(" Link to File : ");
            flag := TRUE;
          ELSIF entry = 4 THEN
            InOut.WriteString(" Link to Directory : ");
            flag := TRUE;
          ELSIF entry = -3 THEN
            InOut.WriteString(" File : ");
            flag := TRUE;
          ELSIF entry = 3 THEN
            InOut.WriteString(" Directory : ");
            flag := TRUE;
          ELSE
            InOut.WriteString(" unkown DirEntryType");InOut.WriteLn;InOut.WriteLn;
            Arts.returnVal := Dos.error;
            Arts.Terminate(Arts.CurrentLevel());
          END;
          *)

          flag := TRUE;
          IF flag THEN
            (*InOut.WriteString(fileName);InOut.WriteLn;*)
            (*IF ABS(entry) = 4 THEN*)
            IF TRUE THEN (* <========== *)
              FileSystem.Lookup(file,linkName,30,TRUE);
              IF (file.res = done) THEN
                (*
                InOut.WriteString("fileName = ");
                InOut.WriteString(fileName);InOut.WriteLn;
                *)
                FileSystem.WriteBytes(file,SYSTEM.ADR(fileName),
                                      Str.Length(fileName),act);
                IF LONGINT(Str.Length(fileName)) # act THEN
                  InOut.WriteString(" Error while writing");InOut.WriteLn;InOut.WriteLn;
                  Arts.returnVal := Dos.error;
                  Arts.Terminate(Arts.CurrentLevel());
                (*
                ELSE
                  InOut.WriteString(" Write ok");InOut.WriteLn;
                  *)
                END;
                FileSystem.Close(file);
                (*
                IF (file.res = done) THEN
                  InOut.WriteString(" Datei geschlossen");InOut.WriteLn;
                END;
                *)
              ELSE
                InOut.WriteString("Fehler bei Open");InOut.WriteLn;
                Arts.returnVal := Dos.error;
                Arts.Terminate(Arts.CurrentLevel());
              END;
            END;
          END;
        ELSE
          InOut.WriteString("Examine not successful");InOut.WriteLn;
          Arts.returnVal := Dos.error;
          Arts.Terminate(Arts.CurrentLevel());
        END;
      ELSE
        InOut.WriteString("Datei nicht gefunden");InOut.WriteLn;
        Arts.returnVal := Dos.fail;
        Arts.Terminate(Arts.CurrentLevel());
      END;
    END;
  END;
END CheckLink;

BEGIN
  Arts.TermProcedure(CleanUp);
  myLock := NIL;
  CheckLink;
END IfLink.


